@@ -1,17 +1,22 @@
-use super::super::FileDbParams;
-use super::dbxxx::FileDbXxxInnerKT;
+use super::super::super::DbXxxKeyType;
+use super::super::{FileBufSizeParam, FileDbParams};
 use super::semtype::*;
 use super::vfile::VarFile;
+use rabuf::SmallRead;
 use std::cell::RefCell;
 use std::convert::TryInto;
 use std::fs::OpenOptions;
-use std::io::{Read, Result, Seek, SeekFrom, Write};
+use std::io::{Read, Result, Write};
 use std::path::Path;
 use std::rc::Rc;
 
 type HeaderSignature = [u8; 8];
 
-const _DAT_HEADER_SZ: u64 = 128;
+const CHUNK_SIZE: u32 = 4 * 1024;
+//const CHUNK_SIZE: u32 = 16 * 4 * 1024;
+//const CHUNK_SIZE: u32 = 1024 * 1024;
+//const CHUNK_SIZE: u32 = 16 * 1024 * 1024;
+const _DAT_HEADER_SZ: u64 = 192;
 const DAT_HEADER_SIGNATURE: HeaderSignature = [b's', b'i', b'a', b'm', b'd', b'b', b'0', 0u8];
 
 #[cfg(not(feature = "record_cache"))]
@@ -22,16 +27,16 @@ use super::rc::RecordCache;
 
 #[cfg(not(feature = "record_cache"))]
 #[derive(Debug)]
-struct VarFileRecordCache<KT: FileDbXxxInnerKT>(VarFile, PhantomData<KT>);
+struct VarFileRecordCache<KT: DbXxxKeyType>(VarFile, PhantomData<KT>);
 
 #[cfg(feature = "record_cache")]
 #[derive(Debug)]
-struct VarFileRecordCache<KT: FileDbXxxInnerKT>(VarFile, RecordCache<KT>);
+struct VarFileRecordCache<KT: DbXxxKeyType>(VarFile, RecordCache<KT>);
 
 #[derive(Debug, Clone)]
-pub struct DatFile<KT: FileDbXxxInnerKT>(Rc<RefCell<VarFileRecordCache<KT>>>);
+pub struct DatFile<KT: DbXxxKeyType>(Rc<RefCell<VarFileRecordCache<KT>>>);
 
-impl<KT: FileDbXxxInnerKT> DatFile<KT> {
+impl<KT: DbXxxKeyType> DatFile<KT> {
     pub fn open_with_params<P: AsRef<Path>>(
         path: P,
         ks_name: &str,
@@ -45,14 +50,24 @@ impl<KT: FileDbXxxInnerKT> DatFile<KT> {
             .write(true)
             .create(true)
             .open(pb)?;
-        let mut file = VarFile::with_capacity(
-            std_file,
-            params.dat_buf_num_chunks,
-            params.dat_buf_chunk_size,
-        )?;
-        let _ = file.seek(SeekFrom::End(0))?;
-        let len = file.stream_position()?;
-        if len == 0 {
+        let mut file = match params.dat_buf_size {
+            FileBufSizeParam::Size(val) => {
+                let dat_buf_chunk_size = CHUNK_SIZE;
+                let dat_buf_num_chunks = val / dat_buf_chunk_size;
+                VarFile::with_capacity(
+                    "dat",
+                    std_file,
+                    dat_buf_chunk_size,
+                    dat_buf_num_chunks.try_into().unwrap(),
+                )?
+            }
+            FileBufSizeParam::PerMille(val) => {
+                VarFile::with_per_mille("dat", std_file, CHUNK_SIZE, val)?
+            }
+            FileBufSizeParam::Auto => VarFile::new("dat", std_file)?,
+        };
+        let file_length: RecordOffset = file.seek_to_end()?;
+        if file_length.is_zero() {
             file.write_recf_init_header(sig2)?;
         } else {
             file.check_recf_header(sig2)?;
@@ -65,18 +80,26 @@ impl<KT: FileDbXxxInnerKT> DatFile<KT> {
         //
         Ok(Self(Rc::new(RefCell::new(file_rc))))
     }
+    #[inline]
+    pub fn read_fill_buffer(&self) -> Result<()> {
+        let mut locked = RefCell::borrow_mut(&self.0);
+        locked.0.read_fill_buffer()
+    }
+    #[inline]
     pub fn flush(&self) -> Result<()> {
         let mut locked = self.0.borrow_mut();
         #[cfg(feature = "record_cache")]
         locked.flush_record_cache()?;
         locked.0.flush()
     }
+    #[inline]
     pub fn sync_all(&self) -> Result<()> {
         let mut locked = self.0.borrow_mut();
         #[cfg(feature = "record_cache")]
         locked.flush_record_cache_clear()?;
         locked.0.sync_all()
     }
+    #[inline]
     pub fn sync_data(&self) -> Result<()> {
         let mut locked = self.0.borrow_mut();
         #[cfg(feature = "record_cache")]
@@ -84,31 +107,53 @@ impl<KT: FileDbXxxInnerKT> DatFile<KT> {
         locked.0.sync_data()
     }
     #[cfg(feature = "buf_stats")]
+    #[inline]
     pub fn buf_stats(&self) -> Vec<(String, i64)> {
         let locked = self.0.borrow();
         locked.0.buf_stats()
     }
     //
+    #[inline]
     pub(crate) fn read_record_only_size(&self, offset: RecordOffset) -> Result<RecordSize> {
         let mut locked = self.0.borrow_mut();
         locked.read_record_only_size(offset)
     }
+    #[inline]
+    pub fn read_record_only_key_length(&self, offset: RecordOffset) -> Result<KeyLength> {
+        let mut locked = self.0.borrow_mut();
+        locked.read_record_only_key_length(offset)
+    }
+    #[inline]
+    pub fn read_record_only_value_length(&self, offset: RecordOffset) -> Result<ValueLength> {
+        let mut locked = self.0.borrow_mut();
+        locked.read_record_only_value_length(offset)
+    }
+    #[inline]
     pub fn read_record_only_key(&self, offset: RecordOffset) -> Result<KT> {
         let mut locked = self.0.borrow_mut();
         locked.read_record_only_key(offset)
     }
+    #[inline]
+    pub fn read_record_only_value(&self, offset: RecordOffset) -> Result<Vec<u8>> {
+        let mut locked = self.0.borrow_mut();
+        locked.read_record_only_value(offset)
+    }
+    #[inline]
     pub fn read_record(&self, offset: RecordOffset) -> Result<Record<KT>> {
         let mut locked = self.0.borrow_mut();
         locked.read_record(offset)
     }
+    #[inline]
     pub fn write_record(&self, record: Record<KT>) -> Result<Record<KT>> {
         let mut locked = self.0.borrow_mut();
         locked.write_record(record, false)
     }
+    #[inline]
     pub fn delete_record(&self, offset: RecordOffset) -> Result<RecordSize> {
         let mut locked = self.0.borrow_mut();
         locked.delete_record(offset)
     }
+    #[inline]
     pub fn add_record(&self, key: &KT, value: &[u8]) -> Result<Record<KT>> {
         let mut locked = self.0.borrow_mut();
         locked.add_record(key, value)
@@ -116,7 +161,7 @@ impl<KT: FileDbXxxInnerKT> DatFile<KT> {
 }
 
 // for debug
-impl<KT: FileDbXxxInnerKT> DatFile<KT> {
+impl<KT: DbXxxKeyType> DatFile<KT> {
     pub fn count_of_free_record(&self) -> Result<Vec<(u32, u64)>> {
         let sz_ary = REC_SIZE_ARY;
         //
@@ -137,25 +182,20 @@ write initiale header to file.
 
 ## header map
 
-The db data header size is 128 bytes.
+The db data header size is 192 bytes.
 
 ```text
 +--------+-------+-------------+---------------------------+
 | offset | bytes | name        | comment                   |
 +--------+-------+-------------+---------------------------+
-| 0      | 4     | signature1  | [b's', b'h', b'a', b'm']  |
-| 4      | 4     | signature1  | [b'd', b'b', b'1', 0u8]   |
+| 0      | 8     | signature1  | [b"siamdb0\0"]            |
 | 8      | 8     | signature2  | 8 bytes type signature    |
 | 16     | 8     | reserve0    |                           |
-| 24     | 8     | free1 off   | offset of free 1st list   |
-| 32     | 8     | free2 off   | offset of free 2ndlist    |
-| 40     | 8     | free3 off   | offset of free 3rd list   |
-| 48     | 8     | free4 off   | offset of free 4th list   |
-| 56     | 8     | free5 off   | offset of free 5th list   |
-| 64     | 8     | free6 off   | offset of free 6th list   |
-| 72     | 8     | free7 off   | offset of free 7th list   |
-| 80     | 8     | free8 off   | offset of free 8th list   |
-| 88     | 40    | reserve1    |                           |
+| 24     | 8     | reserve1    |                           |
+| 32     | 8     | free1 off   | offset of free 1st list   |
+| ...    | ...   | ...         | ...                       |
+| 152    | 8     | free16 off  | offset of free 16th list  |
+| 160    | 32    | reserve2    |                           |
 +--------+-------+-------------+---------------------------+
 ```
 
@@ -166,43 +206,45 @@ The db data header size is 128 bytes.
 
 impl VarFile {
     fn write_recf_init_header(&mut self, signature2: HeaderSignature) -> Result<()> {
-        let _ = self.seek(SeekFrom::Start(0))?;
+        self.seek_from_start(RecordOffset::new(0))?;
         // signature1
         self.write_all(&DAT_HEADER_SIGNATURE)?;
         // signature2
         self.write_all(&signature2)?;
         // reserve0
         self.write_u64_le(0)?;
-        // free1 .. rserve1
-        self.write_all(&[0u8; 104])?;
+        // reserve1
+        self.write_u64_le(0)?;
+        // free1 .. reserve2
+        self.write_all(&[0u8; 160])?;
         //
         Ok(())
     }
     fn check_recf_header(&mut self, signature2: HeaderSignature) -> Result<()> {
-        let _ = self.seek(SeekFrom::Start(0))?;
+        self.seek_from_start(RecordOffset::new(0))?;
         // signature1
         let mut sig1 = [0u8, 0u8, 0u8, 0u8, 0u8, 0u8, 0u8, 0u8];
         let _sz = self.read_exact(&mut sig1)?;
-        assert!(!(sig1 != DAT_HEADER_SIGNATURE), "invalid header signature1");
+        assert!(sig1 == DAT_HEADER_SIGNATURE, "invalid header signature1");
         // signature2
         let mut sig2 = [0u8, 0u8, 0u8, 0u8, 0u8, 0u8, 0u8, 0u8];
         let _sz = self.read_exact(&mut sig2)?;
         assert!(
-            !(sig2 != signature2),
+            sig2 == signature2,
             "invalid header signature2, type signature: {:?}",
             sig2
         );
         // reserve0
         let _reserve0 = self.read_u64_le()?;
-        assert!(!(_reserve0 != 0), "invalid reserve0");
+        assert!(_reserve0 == 0, "invalid reserve0");
         //
         Ok(())
     }
 }
 
-const REC_SIZE_FREE_OFFSET_1ST: u64 = 24;
+const REC_SIZE_FREE_OFFSET_1ST: u64 = 32;
 
-const REC_SIZE_FREE_OFFSET: [u64; 8] = [
+const REC_SIZE_FREE_OFFSET: [u64; 16] = [
     REC_SIZE_FREE_OFFSET_1ST,
     REC_SIZE_FREE_OFFSET_1ST + 8,
     REC_SIZE_FREE_OFFSET_1ST + 8 * 2,
@@ -211,10 +253,35 @@ const REC_SIZE_FREE_OFFSET: [u64; 8] = [
     REC_SIZE_FREE_OFFSET_1ST + 8 * 5,
     REC_SIZE_FREE_OFFSET_1ST + 8 * 6,
     REC_SIZE_FREE_OFFSET_1ST + 8 * 7,
+    REC_SIZE_FREE_OFFSET_1ST + 8 * 8,
+    REC_SIZE_FREE_OFFSET_1ST + 8 * 9,
+    REC_SIZE_FREE_OFFSET_1ST + 8 * 10,
+    REC_SIZE_FREE_OFFSET_1ST + 8 * 11,
+    REC_SIZE_FREE_OFFSET_1ST + 8 * 12,
+    REC_SIZE_FREE_OFFSET_1ST + 8 * 13,
+    REC_SIZE_FREE_OFFSET_1ST + 8 * 14,
+    REC_SIZE_FREE_OFFSET_1ST + 8 * 15,
 ];
 
-pub(crate) const REC_SIZE_ARY: [u32; 8] =
-    [8 * 2, 8 * 3, 8 * 4, 8 * 6, 8 * 8, 8 * 32, 8 * 64, 8 * 256];
+pub(crate) const REC_SIZE_ARY: [u32; 16] = [
+    8 * 2,
+    8 * 3,
+    8 * 4,
+    8 * 6,
+    8 * 8,
+    8 * 10,
+    8 * 12,
+    8 * 14,
+    8 * 8 * 2,
+    8 * 8 * 4,
+    8 * 8 * 6,
+    8 * 8 * 8,
+    8 * 8 * 10,
+    8 * 8 * 12,
+    8 * 8 * 14,
+    8 * 8 * 8 * 2,
+];
+//    [8 * 2, 8 * 3, 8 * 4, 8 * 6, 8 * 8, 8 * 32, 8 * 64, 8 * 256];
 
 impl RecordSize {
     pub(crate) fn is_valid(&self) -> bool {
@@ -261,7 +328,7 @@ impl RecordSize {
                 return RecordSize::new(n_sz);
             }
         }
-        RecordSize::new(((record_size + 511) / 512) * 512)
+        RecordSize::new(((record_size + 128) / 128) * 128)
     }
 }
 
@@ -270,9 +337,8 @@ impl VarFile {
         &mut self,
         record_size: RecordSize,
     ) -> Result<RecordOffset> {
-        let _ = self.seek(SeekFrom::Start(
-            record_size.free_record_list_offset_of_header(),
-        ))?;
+        let free_offset = record_size.free_record_list_offset_of_header();
+        self.seek_from_start(RecordOffset::new(free_offset))?;
         self.read_u64_le().map(RecordOffset::new)
     }
 
@@ -281,10 +347,9 @@ impl VarFile {
         record_size: RecordSize,
         offset: RecordOffset,
     ) -> Result<()> {
-        let _ = self.seek(SeekFrom::Start(
-            record_size.free_record_list_offset_of_header(),
-        ))?;
-        self.write_u64_le(offset.as_value())
+        let free_offset = record_size.free_record_list_offset_of_header();
+        self.seek_from_start(RecordOffset::new(free_offset))?;
+        self.write_u64_le(offset.into())
     }
 
     fn count_of_free_record_list(&mut self, new_record_size: RecordSize) -> Result<u64> {
@@ -295,8 +360,7 @@ impl VarFile {
             while !free_next_offset.is_zero() {
                 count += 1;
                 free_next_offset = {
-                    let _a = self.seek_from_start(free_next_offset)?;
-                    debug_assert!(_a == free_next_offset);
+                    self.seek_from_start(free_next_offset)?;
                     let _record_size = self.read_record_size()?;
                     let _key_len = self.read_key_len()?;
                     debug_assert!(_key_len.is_zero());
@@ -312,7 +376,7 @@ impl VarFile {
         if !new_record_size.is_large_record_size() {
             if !free_1st.is_zero() {
                 let free_next = {
-                    let _ = self.seek_from_start(free_1st)?;
+                    self.seek_from_start(free_1st)?;
                     let (free_next, record_size) = {
                         let record_size = self.read_record_size()?;
                         let _key_len = self.read_key_len()?;
@@ -341,7 +405,7 @@ impl VarFile {
         let mut free_prev = RecordOffset::new(0);
         let mut free_curr = free_1st;
         while !free_curr.is_zero() {
-            let _ = self.seek_from_start(free_curr)?;
+            self.seek_from_start(free_curr)?;
             let (free_next, record_size) = {
                 let record_size = self.read_record_size()?;
                 let _key_len = self.read_key_len()?;
@@ -351,7 +415,7 @@ impl VarFile {
             };
             if new_record_size >= record_size {
                 if !free_prev.is_zero() {
-                    let _ = self.seek_from_start(free_prev)?;
+                    self.seek_from_start(free_prev)?;
                     let _record_size = self.read_record_size()?;
                     let _key_len = self.read_key_len()?;
                     debug_assert!(_key_len.is_zero());
@@ -382,11 +446,10 @@ impl VarFile {
         let free_1st = self.read_free_record_offset_on_header(old_record_size)?;
         {
             let start_offset = self.seek_from_start(old_record_offset)?;
-            debug_assert!(start_offset == old_record_offset);
             self.write_record_size(old_record_size)?;
             self.write_key_len(KeyLength::new(0))?;
             self.write_free_record_offset(free_1st)?;
-            self.write_zero_to(start_offset.as_value() + old_record_size.as_value() as u64)?;
+            self.write_zero_to_offset(start_offset + old_record_size)?;
         }
         self.write_free_record_offset_on_header(old_record_size, old_record_offset)?;
         Ok(())
@@ -394,7 +457,7 @@ impl VarFile {
 }
 
 #[derive(Debug, Default, Clone)]
-pub struct Record<KT: FileDbXxxInnerKT> {
+pub struct Record<KT: DbXxxKeyType> {
     /// offset of IdxNode in dat file.
     pub offset: RecordOffset,
     /// size in bytes of Record in dat file.
@@ -405,7 +468,8 @@ pub struct Record<KT: FileDbXxxInnerKT> {
     pub value: Vec<u8>,
 }
 
-impl<KT: FileDbXxxInnerKT> Record<KT> {
+impl<KT: DbXxxKeyType> Record<KT> {
+    #[inline]
     pub fn with(offset: RecordOffset, size: RecordSize, key: KT, value: Vec<u8>) -> Self {
         Self {
             offset,
@@ -414,6 +478,7 @@ impl<KT: FileDbXxxInnerKT> Record<KT> {
             value,
         }
     }
+    #[inline]
     pub fn with_key_value(key: KT, value: &[u8]) -> Self {
         Self {
             key,
@@ -441,7 +506,7 @@ impl<KT: FileDbXxxInnerKT> Record<KT> {
             let enc_val_len = vu64::encoded_len(value_len.as_value() as u64) as u32;
             let record_len: u32 =
                 enc_key_len + key_len.as_value() + enc_val_len + value_len.as_value();
-            let encorded_record_len = vu64::encoded_len(record_len as u64) as u32;
+            let encorded_record_len = vu64::encoded_len((record_len as u64 + 7) / 8) as u32;
             (encorded_record_len, record_len)
         };
         //
@@ -456,9 +521,7 @@ impl<KT: FileDbXxxInnerKT> Record<KT> {
         let value = &self.value;
         let value_len = ValueLength::new(value.len().try_into().unwrap());
         //
-        let _a = file.seek_from_start(self.offset)?;
-        debug_assert!(_a == self.offset);
-        //
+        file.seek_from_start(self.offset)?;
         file.write_record_size(self.size)?;
         file.write_key_len(key_len)?;
         file.write_all(&key)?;
@@ -470,14 +533,16 @@ impl<KT: FileDbXxxInnerKT> Record<KT> {
     }
 }
 
-impl<KT: FileDbXxxInnerKT> VarFileRecordCache<KT> {
+impl<KT: DbXxxKeyType> VarFileRecordCache<KT> {
     #[cfg(feature = "record_cache")]
+    #[inline]
     fn flush_record_cache(&mut self) -> Result<()> {
         self.1.flush(&mut self.0)?;
         Ok(())
     }
 
     #[cfg(feature = "record_cache")]
+    #[inline]
     fn flush_record_cache_clear(&mut self) -> Result<()> {
         self.1.clear(&mut self.0)?;
         Ok(())
@@ -486,7 +551,7 @@ impl<KT: FileDbXxxInnerKT> VarFileRecordCache<KT> {
     fn delete_record(&mut self, offset: RecordOffset) -> Result<RecordSize> {
         #[cfg(not(feature = "record_cache"))]
         let old_record_size = {
-            let _ = self.0.seek_from_start(offset)?;
+            self.0.seek_from_start(offset)?;
             self.0.read_record_size()?
         };
         #[cfg(feature = "record_cache")]
@@ -494,7 +559,7 @@ impl<KT: FileDbXxxInnerKT> VarFileRecordCache<KT> {
             match self.1.delete(&offset) {
                 Some(record_size) => record_size,
                 None => {
-                    let _ = self.0.seek_from_start(offset)?;
+                    self.0.seek_from_start(offset)?;
                     self.0.read_record_size()?
                 }
             }
@@ -504,6 +569,7 @@ impl<KT: FileDbXxxInnerKT> VarFileRecordCache<KT> {
         Ok(old_record_size)
     }
 
+    #[inline]
     fn add_record(&mut self, key: &KT, value: &[u8]) -> Result<Record<KT>> {
         self.write_record(Record::with_key_value(key.clone(), value), true)
     }
@@ -517,7 +583,7 @@ impl<KT: FileDbXxxInnerKT> VarFileRecordCache<KT> {
         if !is_new {
             #[cfg(not(feature = "record_cache"))]
             let old_record_size = {
-                let _ = self.0.seek_from_start(record.offset)?;
+                self.0.seek_from_start(record.offset)?;
                 self.0.read_record_size()?
             };
             #[cfg(feature = "record_cache")]
@@ -525,7 +591,7 @@ impl<KT: FileDbXxxInnerKT> VarFileRecordCache<KT> {
                 if let Some(record_size) = self.1.get_record_size(&record.offset) {
                     record_size
                 } else {
-                    let _ = self.0.seek_from_start(record.offset)?;
+                    self.0.seek_from_start(record.offset)?;
                     self.0.read_record_size()?
                 }
             };
@@ -534,7 +600,7 @@ impl<KT: FileDbXxxInnerKT> VarFileRecordCache<KT> {
                 // over writes.
                 #[cfg(not(feature = "record_cache"))]
                 {
-                    let _ = self.0.seek_from_start(record.offset)?;
+                    self.0.seek_from_start(record.offset)?;
                     record.size = old_record_size;
                     record.dat_write_record_one(&mut self.0)?;
                     return Ok(record);
@@ -557,11 +623,10 @@ impl<KT: FileDbXxxInnerKT> VarFileRecordCache<KT> {
         {
             let free_record_offset = self.0.pop_free_record_list(new_record_size)?;
             let new_record_offset = if !free_record_offset.is_zero() {
-                let _ = self.0.seek_from_start(free_record_offset)?;
+                self.0.seek_from_start(free_record_offset)?;
                 free_record_offset
             } else {
-                let _ = self.0.seek(SeekFrom::End(0))?;
-                RecordOffset::new(self.0.stream_position()?)
+                self.0.seek_to_end()?
             };
             record.offset = new_record_offset;
             record.size = new_record_size;
@@ -570,7 +635,7 @@ impl<KT: FileDbXxxInnerKT> VarFileRecordCache<KT> {
                 Ok(()) => (),
                 Err(err) => {
                     // recover on error
-                    let _ = self.0.set_len(new_record_offset);
+                    let _ = self.0.set_file_length(new_record_offset);
                     return Err(err);
                 }
             }
@@ -588,23 +653,18 @@ impl<KT: FileDbXxxInnerKT> VarFileRecordCache<KT> {
             return Ok(rc.as_ref().clone());
         }
         //
-        let _ = self.0.seek_from_start(offset)?;
+        self.0.seek_from_start(offset)?;
         let record_size = self.0.read_record_size()?;
         debug_assert!(record_size.is_valid());
         let key_len = self.0.read_key_len()?;
-        let key = if key_len.is_zero() {
-            Vec::with_capacity(0)
-        } else {
-            let mut key = vec![0u8; key_len.try_into().unwrap()];
-            let _ = self.0.read_exact(&mut key)?;
-            key
-        };
+        let maybe = self.0.read_exact_maybeslice(key_len.into())?;
+        let key = KT::from(&maybe);
         //
         let val_len = self.0.read_value_len()?;
-        let mut value = vec![0u8; val_len.try_into().unwrap()];
-        let _ = self.0.read_exact(&mut value)?;
+        let maybe_slice = self.0.read_exact_maybeslice(val_len.into())?;
+        let value = maybe_slice.to_vec();
         //
-        let record = Record::with(offset, record_size, KT::from(&key), value.to_vec());
+        let record = Record::with(offset, record_size, key, value);
         //
         #[cfg(feature = "record_cache")]
         let record = self.1.put(&mut self.0, record, record_size, false)?;
@@ -612,49 +672,102 @@ impl<KT: FileDbXxxInnerKT> VarFileRecordCache<KT> {
         Ok(record)
     }
 
-    fn read_record_only_key(&mut self, offset: RecordOffset) -> Result<KT> {
-        debug_assert!(!offset.is_zero());
-        //
-        #[cfg(feature = "record_cache")]
-        {
-            let record = self.read_record(offset)?;
-            return Ok(record.key.clone());
-        }
-        //
-        #[cfg(not(feature = "record_cache"))]
-        {
-            let _ = self.0.seek_from_start(offset)?;
-            let _record_size = self.0.read_record_size()?;
-            let key_len = self.0.read_key_len()?;
-            let key = if key_len.is_zero() {
-                Vec::with_capacity(0)
-            } else {
-                let mut key = vec![0u8; key_len.try_into().unwrap()];
-                let _ = self.0.read_exact(&mut key)?;
-                key
-            };
-            //
-            Ok(KT::from(&key))
-        }
-    }
-
+    #[inline]
     fn read_record_only_size(&mut self, offset: RecordOffset) -> Result<RecordSize> {
         debug_assert!(!offset.is_zero());
         //
         #[cfg(feature = "record_cache")]
         {
             let record = self.read_record(offset)?;
-            return Ok(record.size);
+            Ok(record.size)
         }
         //
         #[cfg(not(feature = "record_cache"))]
         {
-            let _ = self.0.seek_from_start(offset)?;
+            self.0.seek_from_start(offset)?;
             let record_size = self.0.read_record_size()?;
             //
             Ok(record_size)
         }
     }
+
+    #[inline]
+    fn read_record_only_key_length(&mut self, offset: RecordOffset) -> Result<KeyLength> {
+        debug_assert!(!offset.is_zero());
+        //
+        #[cfg(feature = "record_cache")]
+        {
+            let record = self.read_record(offset)?;
+            Ok(KeyLength::new(record.key.len()))
+        }
+        //
+        #[cfg(not(feature = "record_cache"))]
+        {
+            self.0.seek_skip_to_record_key(offset)?;
+            let key_len = self.0.read_key_len()?;
+            Ok(key_len)
+        }
+    }
+
+    #[inline]
+    fn read_record_only_value_length(&mut self, offset: RecordOffset) -> Result<ValueLength> {
+        debug_assert!(!offset.is_zero());
+        //
+        #[cfg(feature = "record_cache")]
+        {
+            let record = self.read_record(offset)?;
+            Ok(ValueLength::new(record.value.len()))
+        }
+        //
+        #[cfg(not(feature = "record_cache"))]
+        {
+            self.0.seek_skip_to_record_value(offset)?;
+            let val_len = self.0.read_value_len()?;
+            //
+            Ok(val_len)
+        }
+    }
+
+    #[inline]
+    fn read_record_only_key(&mut self, offset: RecordOffset) -> Result<KT> {
+        debug_assert!(!offset.is_zero());
+        //
+        #[cfg(feature = "record_cache")]
+        {
+            let record = self.read_record(offset)?;
+            Ok(record.key)
+        }
+        //
+        #[cfg(not(feature = "record_cache"))]
+        {
+            self.0.seek_skip_to_record_key(offset)?;
+            let key_len = self.0.read_key_len()?;
+            let maybe_slice = self.0.read_exact_maybeslice(key_len.into())?;
+            Ok(KT::from(&maybe_slice))
+        }
+    }
+
+    #[inline]
+    fn read_record_only_value(&mut self, offset: RecordOffset) -> Result<Vec<u8>> {
+        debug_assert!(!offset.is_zero());
+        //
+        #[cfg(feature = "record_cache")]
+        {
+            let record = self.read_record(offset)?;
+            Ok(record.value)
+        }
+        //
+        #[cfg(not(feature = "record_cache"))]
+        {
+            self.0.seek_skip_to_record_value(offset)?;
+            //
+            let val_len = self.0.read_value_len()?;
+            let maybe_slice = self.0.read_exact_maybeslice(val_len.into())?;
+            let value = maybe_slice.to_vec();
+            //
+            Ok(value)
+        }
+    }
 }
 
 /*
